import { CachedPromises } from '../memoize';
import { Package } from '../types';
import { Got } from '../network/got';
export declare type FetchOptions = {
    keypath?: string[];
    linkLocal: boolean;
    force: boolean;
    root: string;
    storePath: string;
    tag: string;
    got: Got;
};
export declare type FetchedPackage = {
    pkg: Package;
    path: string;
    srcPath?: string;
    id: string;
    name: string;
    fromCache: boolean;
    justFetched: boolean;
    firstFetch: boolean;
};
export declare type InstallLog = (msg: string, data?: Object) => void;
/**
 * Installs a package.
 *
 * What it does:
 *
 * - resolve() - resolve from registry.npmjs.org
 * - fetch() - download tarball into node_modules/.store/{name}@{version}
 * - recurse into its dependencies
 * - symlink node_modules/{name}
 *
 * @param {Object} ctx - the context.
 * @param {Object} pkgMeta - meta info about the package to install.
 *
 * @example
 *     install(ctx, 'rimraf@2', './node_modules')
 */
export default function fetch(fetches: CachedPromises<void>, pkgRawSpec: string, modules: string, options: FetchOptions): Promise<FetchedPackage>;
