"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator.throw(value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments)).next());
    });
};
const path = require('path');
const fetch_1 = require('./fetch');
const linkBins_1 = require('./linkBins');
/**
 * Install multiple modules into `modules`.
 *
 * @example
 *     ctx = { }
 *     installMultiple(ctx, { minimatch: '^2.0.0' }, {chokidar: '^1.6.0'}, './node_modules')
 */
function installMultiple(ctx, pkgsMap, modules, options) {
    return __awaiter(this, void 0, void 0, function* () {
        pkgsMap = pkgsMap || {};
        const pkgs = Object.keys(pkgsMap).map(pkgName => getRawSpec(pkgName, pkgsMap[pkgName]));
        ctx.store.packages = ctx.store.packages || {};
        const installedPkgs = (yield Promise.all(pkgs.map(function (pkgRawSpec) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    return yield install(pkgRawSpec, modules, ctx, options);
                }
                catch (err) {
                    if (options.optional) {
                        console.log(`Skipping failed optional dependency ${pkgRawSpec}:`);
                        console.log(err.message || err);
                        return null; // is it OK to return null?  
                    }
                    throw err;
                }
            });
        })))
            .filter(pkg => pkg);
        yield linkBins_1.default(modules);
        return installedPkgs;
    });
}
Object.defineProperty(exports, "__esModule", { value: true });
exports.default = installMultiple;
function install(pkgRawSpec, modules, ctx, options) {
    return __awaiter(this, void 0, void 0, function* () {
        options.keypath = options.keypath || [];
        const fetchedPkg = yield fetch_1.default(ctx.fetches, pkgRawSpec, modules, options);
        const dependency = Object.assign({}, fetchedPkg, {
            keypath: options.keypath,
            dependencies: [],
            optional: options.optional === true,
        });
        addInstalledPkg(ctx.installs, dependency);
        ctx.store.packages[options.dependent] = ctx.store.packages[options.dependent] || {};
        ctx.store.packages[options.dependent].dependencies = ctx.store.packages[options.dependent].dependencies || {};
        // NOTE: the current install implementation
        // does not return enough info for packages that were already installed
        if (dependency.fromCache) {
            return dependency;
        }
        ctx.store.packages[options.dependent].dependencies[dependency.pkg.name] = dependency.id;
        ctx.store.packages[dependency.id] = ctx.store.packages[dependency.id] || {};
        ctx.store.packages[dependency.id].dependents = ctx.store.packages[dependency.id].dependents || [];
        if (ctx.store.packages[dependency.id].dependents.indexOf(options.dependent) === -1) {
            ctx.store.packages[dependency.id].dependents.push(options.dependent);
        }
        // when a package was already installed, update the subdependencies only to the specified depth.
        // justFetched is really just hack to avoid executing installation of subdependencies many times.
        if (!dependency.justFetched && (!dependency.firstFetch || options.keypath.length >= options.depth)) {
            return dependency;
        }
        const nextInstallOpts = Object.assign({}, options, {
            keypath: options.keypath.concat([dependency.id]),
            dependent: dependency.id,
            optional: dependency.optional,
            root: dependency.srcPath,
        });
        dependency.dependencies = Array.prototype.concat.apply([], yield Promise.all([
            installMultiple(ctx, dependency.pkg.dependencies || {}, path.join(dependency.path, 'node_modules'), nextInstallOpts),
            installMultiple(ctx, dependency.pkg.optionalDependencies || {}, path.join(dependency.path, 'node_modules'), Object.assign({}, nextInstallOpts, {
                optional: true
            })),
        ]));
        ctx.piq = ctx.piq || [];
        ctx.piq.push({
            path: dependency.path,
            pkgId: dependency.id
        });
        return dependency;
    });
}
function addInstalledPkg(installs, newPkg) {
    if (!installs[newPkg.id]) {
        installs[newPkg.id] = newPkg;
        return;
    }
    installs[newPkg.id].optional = installs[newPkg.id].optional && newPkg.optional;
}
function getRawSpec(name, version) {
    return version === '*' ? name : `${name}@${version}`;
}
//# sourceMappingURL=installMultiple.js.map